#include "validation.h"

const int MAX_N = 2000;

/* Validate the output of the participant. If the output from our answer starts
 * with "impossible", then we just have to check whether the participant's output
 * is equal to "impossible".
 *
 * If our output does not start with "impossible", then we have to simulate their output.
 *
 * The output in the case that the grid can be unlocked has the following format:
 * l1 c1
 * l2 c2
 * l3 c3
 * ...
 * lT cT
 *
 * Where T is h * w. These represent the coordinates of the cells ordered from the
 * first one that should be taken out to the last one.
 */

// Check if it is possible to solve the gridlock problem (i.e. if the answer is "impossible")
bool read_is_possible(std::ifstream& in) {
    std::string val;
    in >> val;
    return val != "impossible";
}

// This class handles all the logic of the grid from the problem. Use this to simulate
// taking out blocks from the grid.
class Grid {
public:
    // Matrix data. 'h' is the number of rows, 'w' is the number of columns,
    // matr[i][j] contains the arrow from the i'th row, j'th column.
    int h, w;
    std::string matr[MAX_N];
    
    // Create the grid from the input stream
    Grid(std::ifstream& in) {
        in >> h >> w;
        
        for (int l = 0; l < h; ++l) {
            in >> matr[l];
            for (int c = 0; c < w; ++c) {
                taken_out[l][c] = false;

                for (int dir = 0; dir < 4; ++dir) {
                    int ln = l + dl[dir];
                    int cn = c + dc[dir];

                    if (0 <= ln && ln < h && 0 <= cn && cn < w)
                        link_cell[l][c][dir] = 1;
                    else
                        link_cell[l][c][dir] = 0;
                }
            }
        }
    }
    
    int get_number_cells() {
        return h * w;
    }

    // Try to take out a cell from the grid from the given line and column.
    // Returns: (true, "") if taking out the cell was successful
    //          (false, error message) otherwise
    std::pair<bool, std::string> take_out_cell(int l, int c) {
        if (!(0 <= l && l < h && 0 <= c && c < w))
            return {false, "Taking out cell outside the matrix"};

        if (taken_out[l][c])
            return {false, "Cell is already taken out"};
    
        int dir = arrow_to_int(matr[l][c]);
        if (link_cell[l][c][dir] != 0)
            return {false, "Cell is locked"};
        
        erase_cell(l, c);
        return {true, ""};
    }

private:
    /*
     * The way that the solution works is that we don't do the topological
     * sorting, we do it in the other way around. We try taking out pieces
     * from the edge recursively.
     *
     * That's kinda irrelevant for the validator. What is relevant though is 
     * the way that data is stored both here and in the solution. For each
     * cell in the matrix, we store a link to the next cell to its right, up,
     * left and down (basically, a 4-way linked list). We do not hold a pointer,
     * but we hold the distance to the cell in that direction, or 0 if it is 
     * the end of the list.
     *
     * The directions are numbered and described below in the static vectors,
     * there is also a function that converts from a char to a number. If you 
     * want to go to the next cell on the left of the current cell (l, c), you do:
     *
     * dir = arrow_to_int('<');
     * l + dl[dir] * link_cell[l][c][dir]; // The line of the new cell
     * c + dc[dir] * link_cell[l][c][dir]; // The column of the new cell
     */

    int link_cell[MAX_N][MAX_N][4];
    bool taken_out[MAX_N][MAX_N];

    static char constexpr dir_label[4] = {'>', 'v', '<', '^'};
    static int  constexpr dl[4]        = {   0,  1,   0,  -1};
    static int  constexpr dc[4]        = {   1,  0,  -1,   0};

    static int arrow_to_int(char chr) {
        switch (chr) {
            case '>': return 0;
            case 'v': return 1;
            case '<': return 2;
            case '^': return 3;
            default:  return -1;
        }
    }

    // This only works because of the order defined in the static arrays.
    static int get_opposite(int dir) {
        return dir ^ 2;
    }

    // Skip over (l, c)'s next element in the list in the given direction.
    void eliminate_next_from_list(int l, int c, int dir) {
        int ln = l + dl[dir] * link_cell[l][c][dir];
        int cn = c + dc[dir] * link_cell[l][c][dir];

        if (link_cell[ln][cn][dir] == 0)
            link_cell[l][c][dir] = 0;
        else
            link_cell[l][c][dir] += link_cell[ln][cn][dir];
    }

    void erase_cell(int l, int c) {
        taken_out[l][c] = true;

        for (int dir = 0; dir < 4; ++dir) {
            if (link_cell[l][c][dir] != 0) {
                eliminate_next_from_list(
                    l + dl[dir] * link_cell[l][c][dir],
                    c + dc[dir] * link_cell[l][c][dir],
                    get_opposite(dir)
                );
            }
        }
    }
};

int main(int argc, char **argv) {
    // Set up the input and answer streams.
    std::ifstream in(argv[1]);
    std::ifstream ans(argv[2]); // Only for custom checker.
    
    bool possible = read_is_possible(ans);

    OutputValidator v(argc, argv);

    if (possible) {
        if (v.case_sensitive && v.ws && !strstr(argv[2], "/sample/") != NULL) {
            // The generated answer files will only contain the word "possible" to greatly reduce the size of the zip.
            // This works, because the .ans files are not used for validation anyway.
            // We know the output validator is in "generator mode" when it is case- and whitespace-sensitive.
            // Of course, we should not do this for the sample cases.
            // The substring check is only executed on a jury member's computer, when generating the test cases.
            // The substring check is never executed on DOMjudge, because it is always in validation mode there.
            v.test_string("possible");
            v.newline();
            return 0;
        }

        Grid input(in);

        int cnt_cells = input.get_number_cells();
        
        for (int i = 0; i < cnt_cells; ++i) {
            int l = v.read_integer("l", 1, input.h);
            v.space();
            int c = v.read_integer("c", 1, input.w);
            
            v.newline();

            std::pair<bool, std::string> result = input.take_out_cell(l - 1, c - 1);

            if (!result.first)
                v.WA(result.second);
        }
    } else {
        v.test_string("impossible");
        v.newline();
    }

    return 0;
}
