/**
 * For every input pair, there is 1 or 2 ranges that have its value incremented. Then we want to know over a range what
 * minimum value is for any number in a range. The Segment tree is an optimal datastructure for this.
 *
 * My implementation stores the best found min value for a range in the node. It uses lazy propagation and creation
 * only creating nodes if an update doesn't fully cover the range the node of the tree represents with a lazy parameter.
 *
 * Adding all values is O(log n), so adding all ranges is O(m log n). The query is O(1) since it's always the root of
 * the tree, therefor this should be a O(m log n) solution.
 */
fun main() {
    val (n, m) = readln().split(" ").let { (a, b) -> a.toInt() to b.toInt() }
    val tree = SegmentTree(n)
    System.`in`.bufferedReader().lineSequence().take(m)
        .map { l -> l.split(" ").map { it.toInt() }.let { (a, b) -> a to b } }
        .forEach { (w, l) ->
            when {
                w < l -> tree.update(w + 1, l, 1)
                w == l -> Unit // should not happen, bypass for old test cases
                else -> {
                    tree.update(1, l, 1)
                    tree.update(w + 1, n, 1)
                }
            }
        }
    println(tree.query(1, n))
}

private class SegmentTree(size: Int) {
    private val root: SegmentTreeNode
    init {
        root = SegmentTreeNode(1, size)
    }

    fun update(rangeLo: Int, rangeHi: Int, value: Int) = update(rangeLo, rangeHi, value, root)
    fun query(rangeLo: Int, rangeHi: Int) = query(rangeLo, rangeHi, root)

    private fun update(rangeLo: Int, rangeHi: Int, value: Int, node: SegmentTreeNode?) {
        if (node == null) return
        if (rangeLo <= node.lo && node.hi <= rangeHi) {
            node.lazy += value
        }

        pushDown(node)

        // complete overlap or no overlap at all
        if (rangeLo <= node.lo && node.hi <= rangeHi || rangeHi < node.lo || rangeLo > node.hi) return


        update(rangeLo, rangeHi, value, node.left)
        update(rangeLo, rangeHi, value, node.right)

        node.value = minOf(node.left!!.value, node.right!!.value)
    }

    private fun query(rangeLo: Int, rangeHi: Int, node: SegmentTreeNode?): Int {
        if (node == null) return Int.MAX_VALUE
        pushDown(node)

        // no overlap
        if (rangeLo > node.hi || rangeHi < node.lo) return Int.MAX_VALUE

        // complete overlap
        if (rangeLo <= node.lo && node.hi <= rangeHi) return node.value

        val leftMax = query(rangeLo, rangeHi, node.left)
        val rightMax = query(rangeLo, rangeHi, node.right)

        return minOf(leftMax, rightMax)
    }

    private fun pushDown(node: SegmentTreeNode) {
        node.value = node.value + node.lazy

        if (node.lo != node.hi) {
            val mid = node.lo + (node.hi - node.lo) / 2

            if (node.left == null) {
                node.left = SegmentTreeNode(node.lo, mid)
            }

            if (node.right == null) {
                node.right = SegmentTreeNode(mid + 1, node.hi)
            }

            node.left!!.lazy = node.left!!.lazy + node.lazy
            node.right!!.lazy = node.right!!.lazy + node.lazy
        }

        node.lazy = 0
    }
}

private data class SegmentTreeNode(
    val lo: Int,
    val hi: Int,
    var value: Int = 0,
    var lazy: Int = 0,
    var left: SegmentTreeNode? = null,
    var right: SegmentTreeNode? = null
)
